var _url = ((location.protocol==="https:") ? "wss://" : "ws://") + ((location.hostname === "") ? '127.0.0.1' : location.hostname) + ":" + ((location.port === "") ? '9000' : location.port);
var _ws;
var _connected = false;
var _devInfos = null;
var _devNum = -1;
/**
 * 初始化webSocket连接
 */
function ConnectServer(callback, value) {
    if ('WebSocket' in window) {
        _ws = new WebSocket(_url);
    } else if (window.WebSocket) {
        _ws = new WebSocket(_url);
    } else if ('MozWebSocket' in window) {
        _ws = new MozWebSocket(_url);
    } else {
        alert(Lang.WS.lowVer);
    }
    _ws.onopen = function () {
        _connected = true;
        callback(value);
    }
    _ws.onclose = function (e) {
        _connected = false;
    }
    _ws.onmessage = function (e) {
        if (typeof e.data === "string") {
            onTxtMessage(e.data);
        } else {
            onBinMessage(e);
        }
    }
    _ws.onerror = function (e) {
        alert("连接失败");
    };
}


/**
 * 向服务器发送信息
 */
function SendTxt(jsonStr) {
    _connected ? _ws.send(jsonStr) : ConnectServer(SendTxt, jsonStr)
}
function SendJson(json) {
    _connected ? _ws.send(JSON.stringify(json)) : ConnectServer(SendJson, json)
}

/**
 * 处理接收到的服务器文本消息
 */
function onTxtMessage(msg) {
    var jsonObj = JSON.parse(msg);
    if (jsonObj.result != 0) {
        console.log(jsonObj.functionName, jsonObj.errorMsg);
        var info = "";
        if (jsonObj.result) {
            info += "<br>维山错误代码: " + jsonObj.result + ", 错误信息: " + jsonObj.errorMsg;
        }
        displayOutputInfo(3, info);
    } else {
        switch (jsonObj.func) {
            case "pvSignature":
                DisplayImg(jsonObj);
                break;
			case "ReadFingerInfo":
                DisplayFingerprintImg(jsonObj);
                break;
            default:
                displayOutputInfo(1, msg);
                break;
        }
    }
}

// 处理连接错误
function onError(e, msg) {
    displayOutputInfo(3, msg);
}


function navigate(){
    var signURL = document.getElementById("signURL");
    var url = signURL.value;
    var command = JSON.stringify({func: 'navigate',url:url,to:2,reqId: new Date().getTime()});
    SendTxt(command);

}

function openSign(){
    var left = parseInt($("#signleft").val());
    var top = parseInt($("#signtop").val());
    var width = parseInt($("#signwidth").val());
    var height = parseInt($("#signheight").val());
    var background = $("#signbackground").val();
    var pen = $("#signpen").val();
    var remark = $("#signremark").val();
    var command = JSON.stringify({func: 'openSign', sign:"open",remark:remark,color:{background:background,pen:pen},pos:{ left:left, top:top, width:width, height:height}, to:2,reqId: new Date().getTime()});
    SendTxt(command);
}

function closeSign(){
    var command = JSON.stringify({func: 'openSign', sign:"close", to:2, reqId:new Date().getTime()});
    SendTxt(command);

}


function openSignAndFinger(){

    var command = JSON.stringify({func: 'startSignAndGetFingerprint', to:2,reqId: new Date().getTime()});
    SendTxt(command);
}

function displayOutputInfo(disp, s) {
    if (disp > 0 && disp < 4) {
        var ary = ["info", "success", "warning"];
        $('#outInfo').css("display", "").attr("class", "col-md-12 alert alert-" + ary[disp - 1]).html("<b>" + s + "</b>");
    } else {
        $('#outInfo').css("display", "none").html("");
    }
}


//添加图片项
function DisplayImg(obj) {
    var imgBase64Str = obj.imgBase64;

    $("#image_data").empty();
    var h = '<div><img width="360" style="padding:5px;" src="' + 'data:jpg' + ';base64,' + imgBase64Str + '"></div>';
    $("#image_data").prepend(h);
    displayOutputInfo(3, "获取签字成功。remark:" + obj.remark);
}

// 添加指纹项
function DisplayFingerprintImg(obj){
	
	$("#fpImage").remove();
	
    var imgBase64Str = obj.fingerImageBase64;

    var h = '<div id="fpImage"><img width="240" style="padding:5px;" src="' + 'data:jpg' + ';base64,' + imgBase64Str + '"></div>';
    $("#image_data").prepend(h);
    displayOutputInfo(3, "获取指纹成功。remark:" + obj.remark);
	var command = JSON.stringify({func: 'navigate',url:"",to:2,reqId: new Date().getTime()});
    SendTxt(command);
}


$(function() {
    $("#closeWindow").click(function() {
        $("#qrcode").fadeOut("slow");
    })

});

//=============================================================================
//=============================================================================
//=============================================================================
//=============================================================================


// 删除全部拍照文件
function clearImageFile() {
    $("#tabBody").empty();
    $("#image_data").empty();
}


$(window).unload(function () {
    // 关闭页面之前,关闭摄像头,关闭连接
    if (_ws && _ws.connected) {
        _ws.Close();
        _ws = null;
    }
});


