var _ws = new MzWsConnect(location.port, onTxtMessage, onError, ClearOutputInfo);

// 处理服务器文本消息
function onTxtMessage(msg) {
    var v = JSON.parse(msg);
    if (v.result == 0) {
        // 返回result=0表示执行成功,其他值表示异常。
        switch (v.func) {
            case "GetScannerList":
                DisplayDevList(v);
                break;
            case "GetScannerParametes":
                DisplayDevParam(v);
                _ws.SendJson({func: "GetScanStatus", reqId: new Date().getTime()});
            case "GetScanStatus":
                DisplayScanState(v);
                break;
            case "Scan":
                DisplayScanResult(v);
                break;
            case "FileToBase64":
                DisplayFileView(v);
                break;
            case "UploadHttpFile":
                DisplayUploadResult(v);
                break;
            default:
                console.log(msg);
                break;
        }
    } else {
        DisplayOutputInfo(3, "命令: " + v.func + "<br>错误码: " + v.result + "<br>错误描述: " + v.errorMsg);
    }
}

// 处理连接错误
function onError(e, msg) {
    DisplayOutputInfo(3, msg);
}

function ClearOutputInfo() {
    DisplayOutputInfo(0);
}

function DisplayDevList(v) {
    var names = v.scannerNames;
    if (names.length == 0) {
        DisplayOutputInfo(3, "获取扫描仪设备信息时，返回的scannerNames为空.");
        return;
    }

    var obj = document.getElementById("devName");
    var lastSelectedIndex = obj.options.selectedIndex;
    obj.options.length = 0;
    for (var i = 0; i < names.length; i++) {
        // 第一个扫描设备的索引是0，第二个扫描设备的索引是1
        obj.options.add(new Option(names[i], i));
        if (lastSelectedIndex == -1) {
            // 默认使用第一个
            lastSelectedIndex = i;
        }
    }
    if (lastSelectedIndex >= 0 && lastSelectedIndex < names.length) {
        obj.options.selectedIndex = lastSelectedIndex;
        onSelectDev();
    }
}

function DisplayDevParam(v) {
    if (v.colorType) {
        var obj = document.getElementById("colorType");
        obj.options.length = 0;
        for (var key in v.colorType) {
            var item = new Option(v.colorType[key], key);
            if (v.currentColorType == key) {
                item.selected = true;
            }
            obj.options.add(item);
        }
    } else {
        DisplayOutputInfo(3, "返回的扫描仪设备参数中,没有colorType属性");
    }

    if (v.pageSize) {
        var obj = document.getElementById("pageSize");
        obj.options.length = 0;
        for (var key in v.pageSize) {
            var item = new Option(v.pageSize[key], key);
            if (v.currentPageSize == key) {
                item.selected = true;
            }
            obj.options.add(item);
        }
    } else {
        DisplayOutputInfo(3, "返回的扫描仪设备参数中,没有pageSize属性");
    }

    if (v.pageSource) {
        var obj = document.getElementById("pageSource");
        obj.options.length = 0;
        for (var i = 0; i < v.pageSource.length; i++) {
            obj.options.add(new Option(v.pageSource[i], i));
        }
        if (typeof (v.currentPageSource) !== undefined) {
            obj.options.selectedIndex = v.currentPageSource;
        }
    } else {
        DisplayOutputInfo(3, "返回的扫描仪设备参数中,没有pageSource属性");
    }

    if (v.scanDPI) {
        var curDPI = v.currentScanDPI;
        var obj = document.getElementById("scanDPI");
        obj.options.length = 0;
        for (var i = 0; i < v.scanDPI.length; i++) {
            obj.options.add(new Option(v.scanDPI[i], i));
            if (curDPI == v.scanDPI[i]) {
                obj.options.selectedIndex = i;
            }
        }
    } else {
        DisplayOutputInfo(3, "返回的扫描仪设备参数中,没有pageSize属性");
    }

    removeBlank.checked = v.isRemoveBlank;
}

function DisplayScanState(v) {
    // scanning扫描状态: 0表示空闲，1表示正在扫描
    if (v.scanning == 1) {
        $("#btnScan").attr("disabled", true);
        $("#btnScan").text('扫描中...');
        $("#btnStop").attr("disabled", false);
    } else {
        $("#btnScan").attr("disabled", false);
        $("#btnScan").text('开始扫描');
        $("#btnStop").attr("disabled", true);
    }
}

// 上次显示警告信息的时间,防止警告信息被快速盖掉
var lastWarningTime = 0;

function DisplayOutputInfo(disp, s) {
    if (disp > 0 && disp < 4) {
        var ary = ["info", "success", "warning"];
        if (new Date().getTime() - lastWarningTime < 8000) {
            // 之前显示的警告信息不足8秒，新信息加在后面。
            $('#outInfo').append("<br><hr><b>" + s + "</b>");
        } else {
            $('#outInfo').css("display", "").attr("class", "col-md-12 alert alert-" + ary[disp - 1]).html("<b>" + s + "</b>");
        }
        if (disp == 3) {
            lastWarningTime = new Date().getTime();
        }
    } else {
        $('#outInfo').css("display", "none").html("");
        lastWarningTime = 0;
    }
}

// 显示Scan指令的回应消息
function DisplayScanResult(v) {
    // {"event":"OnScannedFile","func":"Scan","imagePath":"D:\\Test\\新建 文件夹\\こんにちは\\05ee0058\\1695807300_0029.jpg","reqId":1695807109219,"result":0,"scanState":0}
    // {"event":"OnScanEnd","func":"Scan","reqId":1695807109219,"result":0,"scanState":1}
    switch (v.event) {
        case "OnScannedFile":
            // 扫描到一个文件（在使用多图裁切时扫描一张，会裁切生成多个文件，会有多次OnScannedFile的Scan结果的消息）
            DisplayFile(v.imagePath);
            break;
        case "OnScanEnd":
            // 扫描结束
            DisplayOutputInfo(2, "扫描已完成");
            $("#btnScan").attr("disabled", false);
            $("#btnScan").text('开始扫描');
            $("#btnStop").attr("disabled", true);
            break;
        default:
            DisplayOutputInfo(2, "扫描启动成功，正在进行扫描...");
            $("#btnScan").attr("disabled", true);
            $("#btnScan").text('扫描中...');
            $("#btnStop").attr("disabled", false);
            break;
    }
}

function DisplayFile(path) {
    var pos = path.lastIndexOf('\\');
    var fileName = path.substring(pos + 1);
    var html = '<tr dataPath="' + path + '"><td><a href="#" onclick="vf()">' + fileName + '</a></td><td><input type="checkbox" onchange="onSelect()"></a></td><td></td></tr>';
    $("#tabBody").append(html);
}

// 弹框显示本地文件
function vf() {
    var e = e || window.event;
    var target = e.target || e.srcElement;
    var file = $(target).parent().parent().attr('dataPath');
    // 把文件转为base64,用于显示查看
    _ws.SendJson({func: 'FileToBase64', filePath: file, reqId: new Date().getTime()});
}

function DisplayFileView(v) {
    document.getElementById("imgView").src = "data:" + v.mime + ";base64," + v.imgBase64Str;
    $('#myModal').modal();
}

function onSelect() {
    var count = $("#tabBody input[type='checkbox']:checked").length;
    $("#btnUpload").attr("disabled", count === 0);
}

// 把文件列表中的照片文件，生成 {File1: "11.jpg", File2: "22.jpg", ......}这样的map对象
// 在HTTP上传文件时，“File1”表示form-data的name, HTTP代码为：Content-Disposition: form-data; name="File1"; filename="11.jpg"
function getImgFileMap() {
    var fileMap = {};
    var trs = $("#tabBody input[type='checkbox']:checked").parent().parent();
    for (i = 0; i < trs.length; i++) {
        fileMap["File" + (i + 1)] = trs[i].getAttribute("dataPath");
    }
    return fileMap;
}

function doUpload() {
    $("#btnUpload").attr("disabled", true);
    var url1 = $("#uploadURL").val();
    var cookieVal = $("#cookieVal").val();
    var param = $("#uploadParams").val();
    var cmd = {func: "UploadHttpFile", url: url1, files: getImgFileMap(), reqId: new Date().getTime()};
    if (cookieVal.length > 0) {
        cmd.cookie = cookieVal;
    }
    if (param.length > 0) {
        cmd.params = JSON.parse(param);
    }
    _ws.SendJson(cmd);
    DisplayOutputInfo(1, '文件HTTP上传中...');
}


function DisplayUploadResult(v) {
    // {
    //     "func": "UploadHttpFile",
    //     "reqId": "1666057336437",
    //     "result": 0,
    //     "responseCode": 200,
    //     "response": "{\"viewUrl\":\"/file/download?f=\",\"result\":\"success\",\"files\":[{\"filedName\":\"File1\",\"originalFilename\":\"1666057281.jpg\",\"serverFileName\":\"1666057337471_0.jpg\"},{\"filedName\":\"File2\",\"originalFilename\":\"1666057285.jpg\",\"serverFileName\":\"1666057337471_0.jpg\"}],\"parameters\":{\"Param2\":[\"value2\"],\"Param1\":[\"value1\"]}}",
    // }
    if (v.responseCode == 200) {
        var jv = JSON.parse(v.response);
        console.log(jv);
        var url1 = $("#uploadURL").val();
        var pos = url1.indexOf('/', 9);
        var host = url1.substr(0, pos);
        var trs = $("#tabBody tr");
        for (var i = 0; i < jv.files.length; i++) {
            var localFile = jv.files[i].originalFilename;
            var remotFile = jv.files[i].serverFileName;
            for (var j = 0; j < trs.length; j++) {
                var tr = trs[j];
                if (tr.children[0].innerText == localFile) {
                    $(tr.children[1]).html("");
                    $(tr.children[2]).html('<a href=\"' + host + jv.viewUrl + remotFile + '\" target=\"_blank\">' + remotFile + '</a>');
                    break;
                }
            }
        }
        DisplayOutputInfo(2, '文件HTTP上传成功');
    } else {
        DisplayOutputInfo(3, "Http上传文件失败：<br>responseCode: " + v.responseCode + "<br>response: " + v.response);
    }
}

function StartScan() {
    $("#btnScan").attr("disabled", true);
    $("#tabBody").empty();
    var sannnerParam = {};
    if (showUI.checked) {
        sannnerParam = {
            devNum: parseInt($("#devName").val()), //扫描设备号，从0开始
            showUI: true // showUI时，忽略pageSource\pageSize\colorType\scanDPI属性
        };
    } else {
        sannnerParam = {
            devNum: parseInt($("#devName").val()),
            pageSource: parseInt($("#pageSource").val()),
            pageSize: parseInt($("#pageSize").val()),
            colorType: parseInt($("#colorType").val()),
            scanDPI: parseInt($("#scanDPI").find("option:selected").text())
        };
    }
    var savePath = $("#savePath").val().trim();
    if (savePath.length > 0) {
        // 扫描文件保存的路径,可缺省
        sannnerParam.savePath = savePath;
    }

    var processParam = {
        cropType: parseInt($("#cropType").val()),
        imageType: parseInt($("#imageType").val()),
        fillBorderType: parseInt($("#fillBorderType").val()),
        removalForeign: parseInt($("#removalForeign").val()),
        removeBlank: removeBlank.checked ? 1 : 0,
        jpgQuanlity: parseInt($("#jpgQuanlity").val()),
    };
    if (watermark.checked) {
        processParam.watermark = {
            font: $("#wmFont").val(),
            fontSize: parseInt($("#wmSize").val()),
            color: $("#wmColor").val(),
            transparent: parseInt($("#wmTransparent").val()),
            text: $("#wmText").val(),
            layout: parseInt($("#wmLayout").val())
        }
    }

    var cmd = {
        func: "Scan",
        reqId: new Date().getTime(),
        mode: "path", // 返回照片的方式: base64; path, 默认为path, 也可同时使用,中间用;或,或&隔开，如: base64;path
        sannnerParam: sannnerParam,
        processParam: processParam
    };
    _ws.SendJson(cmd);
}

function StopScan() {
    if (window.confirm("确认停止当前扫描任务吗？")) {
        _ws.SendJson({func: "Scan", stop: 1, reqId: new Date().getTime()});
    }
}

function onSelectDev() {
    $("#btnScan").attr("disabled", true);
    var scannerIndex = parseInt($("#devName").val());
    _ws.SendJson({func: "GetScannerParametes", devNum: scannerIndex, reqId: new Date().getTime()});
}

function onShowUI() {
    if (showUI.checked) {
        $("#pageSource").attr("disabled", "disabled");
        $("#pageSize").attr("disabled", "disabled");
        $("#colorType").attr("disabled", "disabled");
        $("#scanDPI").attr("disabled", "disabled");
        $("#removeBlank").attr("disabled", "disabled");
        $("#autoCut").attr("disabled", "disabled");
    } else {
        $("#pageSource").removeAttr("disabled");
        $("#pageSize").removeAttr("disabled");
        $("#colorType").removeAttr("disabled");
        $("#scanDPI").removeAttr("disabled");
        $("#removeBlank").removeAttr("disabled");
        $("#autoCut").removeAttr("disabled");
    }
}

function onChgWatermark() {
    if (watermark.checked) {
        $("#wmColor").removeAttr("disabled");
        $("#wmTransparent").removeAttr("disabled");
        $("#wmLayout").removeAttr("disabled");
        $("#wmFont").removeAttr("disabled");
        $("#wmSize").removeAttr("disabled");
        $("#wmText").removeAttr("disabled");
    } else {
        $("#wmColor").attr("disabled", "disabled");
        $("#wmTransparent").attr("disabled", "disabled");
        $("#wmLayout").attr("disabled", "disabled");
        $("#wmFont").attr("disabled", "disabled");
        $("#wmSize").attr("disabled", "disabled");
        $("#wmText").attr("disabled", "disabled");
    }
}

$(function () {
    onChgWatermark();
    $("#btnScan").attr("disabled", true);
    $("#btnStop").attr("disabled", true);
    $("#btnUpload").attr("disabled", true);
    _ws.SendJson({func: "GetScannerList", reqId: new Date().getTime()});
});
